@@ -1,4 +1,4 @@
-# $Id: DBI.pm 559 2009-01-24 00:30:59Z ykerherve $
+# $Id$
 
 package Data::ObjectDriver::Driver::DBI;
 use strict;
@@ -90,17 +90,20 @@ sub init_db {
 sub rw_handle {
     my $driver = shift;
     my $db = shift || 'main';
-    $driver->dbh(undef) if $driver->dbh and !$driver->dbh->ping;
-    my $dbh = $driver->dbh;
-    unless ($dbh) {
-        if (my $getter = $driver->get_dbh) {
-            $dbh = $getter->();
-        } else {
-            $dbh = $driver->init_db($db) or die $driver->last_error;
-            $driver->dbh($dbh);
-        }
+    my $dbh;
+    if ($dbh = $driver->dbh) {
+        return $dbh if $dbh->ping;
+
+        ## ping fails, kill cache.
+        delete $Handles{$driver->dsn};
     }
-    $dbh;
+    if (my $getter = $driver->get_dbh) {
+        $dbh = $getter->();
+    } else {
+        $dbh = $driver->init_db($db) or die $driver->last_error;
+        $driver->dbh($dbh);
+    }
+    return $dbh;
 }
 *r_handle = \&rw_handle;
 
@@ -138,6 +141,11 @@ sub fetch {
     my $driver = shift;
     my($rec, $class, $orig_terms, $orig_args) = @_;
 
+    if ($Data::ObjectDriver::RESTRICT_IO) {
+        use Data::Dumper;
+        die "Attempted DBI I/O while in restricted mode: fetch() " . Dumper($orig_terms, $orig_args);
+    }
+
     my ($sql, $bind, $stmt) = $driver->prepare_fetch($class, $orig_terms, $orig_args);
 
     my @bind;
@@ -275,6 +283,10 @@ sub exists {
     my($obj) = @_;
     return unless $obj->has_primary_key;
 
+    if ($Data::ObjectDriver::RESTRICT_IO) {
+        die "Attempted DBI I/O while in restricted mode: exists()";
+    }
+
     ## should call pre_search trigger so we can use enum in the part of PKs
     my $terms = $obj->primary_key_to_terms;
 
@@ -329,6 +341,10 @@ sub _insert_or_replace {
     my $driver = shift;
     my($orig_obj, $options) = @_;
 
+    if ($Data::ObjectDriver::RESTRICT_IO) {
+        die "Attempted DBI I/O while in restricted mode: _insert_or_replace()";
+    }
+
     ## Syntax switch between INSERT or REPLACE statement based on options
     $options ||= {};
     my $INSERT_OR_REPLACE = $options->{replace} ? 'REPLACE' : 'INSERT';
@@ -379,8 +395,6 @@ sub _insert_or_replace {
     }
     eval { $sth->execute };
 	die "Failed to execute $sql with ".join(", ",@$cols).": $@" if $@;
-    _close_sth($sth);
-    $driver->end_query($sth);
 
     ## Now, if we didn't have an object ID, we need to grab the
     ## newly-assigned ID.
@@ -394,6 +408,9 @@ sub _insert_or_replace {
         $orig_obj->$id_col($id);
     }
 
+    _close_sth($sth);
+    $driver->end_query($sth);
+
     $obj->call_trigger('post_save', $orig_obj);
     $obj->call_trigger('post_insert', $orig_obj);
 
@@ -407,6 +424,11 @@ sub update {
 
     my($orig_obj, $terms) = @_;
 
+    if ($Data::ObjectDriver::RESTRICT_IO) {
+        use Data::Dumper;
+        die "Attempted DBI I/O while in restricted mode: _update() " . Dumper($terms);
+    }
+
     ## Use a duplicate so the pre_save trigger can modify it.
     my $obj = $orig_obj->clone_all;
     $obj->call_trigger('pre_save', $orig_obj);
@@ -488,6 +510,10 @@ sub remove {
 
     return unless $orig_obj->has_primary_key;
 
+    if ($Data::ObjectDriver::RESTRICT_IO) {
+        die "Attempted DBI I/O while in restricted mode: remove()";
+    }
+
     ## Use a duplicate so the pre_save trigger can modify it.
     my $obj = $orig_obj->clone_all;
     $obj->call_trigger('pre_remove', $orig_obj);
@@ -513,6 +539,10 @@ sub direct_remove {
     my $driver = shift;
     my($class, $orig_terms, $orig_args) = @_;
 
+    if ($Data::ObjectDriver::RESTRICT_IO) {
+        die "Attempted DBI I/O while in restricted mode: direct_remove() " . Dumper($orig_terms, $orig_args);
+    }
+
     ## Use (shallow) duplicates so the pre_search trigger can modify them.
     my $terms = defined $orig_terms ? { %$orig_terms } : {};
     my $args  = defined $orig_args  ? { %$orig_args  } : {};
@@ -552,6 +582,10 @@ sub bulk_insert {
     Carp::croak("Driver doesn't support bulk_insert")
 	    unless ($dbd->can('bulk_insert'));
 
+    if ($Data::ObjectDriver::RESTRICT_IO) {
+        die "Attempted DBI I/O while in restricted mode: bulk_insert()";
+    }
+
     # check that cols are valid..
     my %valid_cols = map {$_ => 1} @{$class->column_names};
 
@@ -630,7 +664,7 @@ sub _end_txn {
 
 sub DESTROY {
     my $driver = shift;
-    ## Don't take the responsability of disconnecting this handler
+    ## Don't take the responsibility of disconnecting this handler
     ## if we haven't created it ourself.
     return unless $driver->{__dbh_init_by_driver};
     if (my $dbh = $driver->dbh) {
